% eegplugin_besa1_0() - EEGLAB plugin for finding component source dipole models
%                    using BESA (Brain Electric Source Analysis, Megis Software, 
%                    Munich) version 3.0. (Later versions do not have batch mode
%                    scripting. Version 2.2 may work as well).
% Usage:
%   >> eegplugin_besa1_0(fig);
%   >> eegplugin_besa1_0(fig, trystrs, catchstrs);
%
% Inputs:
%   fig        - [integer] EEGLAB figure
%   trystrs    - [struct]  "try" strings for menu callbacks. See notes. 
%   catchstrs  - [struct]  "catch" strings for menu callbacks. See notes. 
%
% Notes:
%   To create a new plugin, simply create a file beginning with "eegplugin_"
%   and place it in your eeglab folder. It will then be automatically 
%   detected by eeglab. See also this source code internal comments.
%   For eeglab to return errors and add the function's results to 
%   the eeglab history, menu callback must be nested into "try" and 
%   a "catch" strings. For more information on how to create eeglab 
%   plugins, see http://www.sccn.ucsd.edu/eeglab/contrib.html
%
% Author: Arnaud Delorme, CNL / Salk Institute, 22 February 2003
%
% See also: eeglab()

%123456789012345678901234567890123456789012345678901234567890123456789012

% Copyright (C) 2003 Arnaud Delorme, Salk Institute, arno@salk.edu
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1.07  USA

% $Log: eegplugin_besa1_0.m,v $
% Revision 1.7  2003/12/06 00:09:22  arno
% version number
% /
%
% Revision 1.6  2003/12/04 01:37:06  arno
% path
%
% Revision 1.5  2003/12/02 17:34:22  arno
% besa path
%
% Revision 1.4  2003/11/26 03:37:27  arno
% add besa folder to path
%
% Revision 1.3  2003/11/26 03:32:28  arno
% debug last
%
% Revision 1.2  2003/11/26 03:31:24  arno
% add path to plugin
%
% Revision 1.1  2003/11/26 03:18:17  arno
% Initial revision
%
% Revision 1.12  2003/11/25 19:20:35  arno
% same
%
% Revision 1.11  2003/11/25 19:18:44  arno
% conforming to smart plugin
%
% Revision 1.10  2003/08/19 22:25:38  arno
% same
%
% Revision 1.9  2003/08/19 22:24:27  arno
% fixed last modeif
%
% Revision 1.8  2003/08/19 22:22:48  arno
% fixing storing
%
% Revision 1.7  2003/03/12 00:38:41  arno
% error calling dipplot
%
% Revision 1.6  2003/03/12 00:37:47  arno
% new check
%
% Revision 1.5  2003/03/11 23:35:04  arno
% update for pop_dipplot.m
%
% Revision 1.4  2003/02/26 17:03:15  arno
% besaplot -> dipplot
%
% Revision 1.3  2003/02/25 01:11.02  scott
% header edit -sm
%
% Revision 1.2  2003/02/24 19:35:52  arno
% adding besa plugin
%
% Revision 1.1  2003/02/24 19:32:24  arno
% Initial revision
%

function eegplugin_besa1_0(fig, trystrs, catchstrs)
    
    if nargin < 3
        error('eegplugin_besa1_0 requires 3 arguments');
    end;
    
    % add besa folder to path
    % -----------------------
    if ~exist('besaimport')
        p = which('eegplugin_besa1_0');
        p = p(1:findstr(p,'eegplugin_besa1_0.m')-1);
        addpath([ p 'besa1.0' ] );
    end;
    
    % find tools menu
    % ---------------
    menu = findobj(fig, 'tag', 'tools'); 
    % tag can be 
    % 'import data'  -> File > import data menu
    % 'import epoch' -> File > import epoch menu
    % 'import event' -> File > import event menu
    % 'export'       -> File > export
    % 'tools'        -> tools menu
    % 'plot'         -> plot menu
    
    % command to check that the '.source' is present in the EEG structure 
    % -------------------------------------------------------------------
    check_besa = [ 'if ~isfield(EEG, ''sources''), error(''No dipole information in dataset''); end;'  ...
                 trystrs.no_check ];
    
    % menu callback commands
    % ----------------------
    catchstrs.store_and_hist = [ ...
                        '[ALLEEG EEG] = eeg_store(ALLEEG, EEG, CURRENTSET); ' ...
                        'catch, errordlg2(lasterr, ''EEGLAB error''); LASTCOM= ''''; clear EEGTMP; end;' ...
                        'h(LASTCOM); disp(''Done.''); eeglab(''redraw'');' ];
    comexport = [ trystrs.check_ica 'besaexport(EEG);' catchstrs.add_to_hist ]; 
    comimport = [ trystrs.check_ica 'EEG = besaimport(EEG);' catchstrs.store_and_hist ];
    complot1  = [ check_besa 'pop_dipplot(EEG);' catchstrs.add_to_hist ];
    
    % create menus
    % ------------
    submenu = uimenu( menu, 'Label', 'Locate dipoles using BESA');
    uimenu( submenu, 'Label', 'Export components to BESA'     , 'CallBack', comexport);
    uimenu( submenu, 'Label', 'Import dipoles from BESA'      , 'CallBack', comimport);
    uimenu( submenu, 'Label', 'Plot dipoles on BESA head'     , 'CallBack', complot1);
 